###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ..helperfunctions import _xml_to_list
from ...worksheet import Worksheet


class TestAssembleWorksheet(unittest.TestCase):
    """
    Test assembling a complete Worksheet file.

    """

    def test_assemble_xml_file(self):
        """Test writing a worksheet with no cell data."""
        self.maxDiff = None

        fh = StringIO()
        worksheet = Worksheet()
        worksheet._set_filehandle(fh)
        worksheet.select()
        worksheet.name = "Sheet1"
        worksheet.excel_version = 2010

        data = [-2, 2, 3, -1, 0]
        worksheet.write_row("A1", data)

        # Set up sparklines.

        # Test all the styles.
        for i in range(36):
            row = i + 1
            sparkrange = "Sheet1!A%d:E%d" % (row, row)
            worksheet.write_row(i, 0, data)
            worksheet.add_sparkline(i, 5, {"range": sparkrange, "style": row})

        worksheet._assemble_xml_file()

        exp = _xml_to_list(
            """
                <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
                <worksheet xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:mc="http://schemas.openxmlformats.org/markup-compatibility/2006" xmlns:x14ac="http://schemas.microsoft.com/office/spreadsheetml/2009/9/ac" mc:Ignorable="x14ac">
                  <dimension ref="A1:E36"/>
                  <sheetViews>
                    <sheetView tabSelected="1" workbookViewId="0"/>
                  </sheetViews>
                  <sheetFormatPr defaultRowHeight="15" x14ac:dyDescent="0.25"/>
                  <sheetData>
                    <row r="1" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A1">
                        <v>-2</v>
                      </c>
                      <c r="B1">
                        <v>2</v>
                      </c>
                      <c r="C1">
                        <v>3</v>
                      </c>
                      <c r="D1">
                        <v>-1</v>
                      </c>
                      <c r="E1">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="2" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A2">
                        <v>-2</v>
                      </c>
                      <c r="B2">
                        <v>2</v>
                      </c>
                      <c r="C2">
                        <v>3</v>
                      </c>
                      <c r="D2">
                        <v>-1</v>
                      </c>
                      <c r="E2">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="3" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A3">
                        <v>-2</v>
                      </c>
                      <c r="B3">
                        <v>2</v>
                      </c>
                      <c r="C3">
                        <v>3</v>
                      </c>
                      <c r="D3">
                        <v>-1</v>
                      </c>
                      <c r="E3">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="4" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A4">
                        <v>-2</v>
                      </c>
                      <c r="B4">
                        <v>2</v>
                      </c>
                      <c r="C4">
                        <v>3</v>
                      </c>
                      <c r="D4">
                        <v>-1</v>
                      </c>
                      <c r="E4">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="5" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A5">
                        <v>-2</v>
                      </c>
                      <c r="B5">
                        <v>2</v>
                      </c>
                      <c r="C5">
                        <v>3</v>
                      </c>
                      <c r="D5">
                        <v>-1</v>
                      </c>
                      <c r="E5">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="6" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A6">
                        <v>-2</v>
                      </c>
                      <c r="B6">
                        <v>2</v>
                      </c>
                      <c r="C6">
                        <v>3</v>
                      </c>
                      <c r="D6">
                        <v>-1</v>
                      </c>
                      <c r="E6">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="7" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A7">
                        <v>-2</v>
                      </c>
                      <c r="B7">
                        <v>2</v>
                      </c>
                      <c r="C7">
                        <v>3</v>
                      </c>
                      <c r="D7">
                        <v>-1</v>
                      </c>
                      <c r="E7">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="8" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A8">
                        <v>-2</v>
                      </c>
                      <c r="B8">
                        <v>2</v>
                      </c>
                      <c r="C8">
                        <v>3</v>
                      </c>
                      <c r="D8">
                        <v>-1</v>
                      </c>
                      <c r="E8">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="9" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A9">
                        <v>-2</v>
                      </c>
                      <c r="B9">
                        <v>2</v>
                      </c>
                      <c r="C9">
                        <v>3</v>
                      </c>
                      <c r="D9">
                        <v>-1</v>
                      </c>
                      <c r="E9">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="10" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A10">
                        <v>-2</v>
                      </c>
                      <c r="B10">
                        <v>2</v>
                      </c>
                      <c r="C10">
                        <v>3</v>
                      </c>
                      <c r="D10">
                        <v>-1</v>
                      </c>
                      <c r="E10">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="11" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A11">
                        <v>-2</v>
                      </c>
                      <c r="B11">
                        <v>2</v>
                      </c>
                      <c r="C11">
                        <v>3</v>
                      </c>
                      <c r="D11">
                        <v>-1</v>
                      </c>
                      <c r="E11">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="12" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A12">
                        <v>-2</v>
                      </c>
                      <c r="B12">
                        <v>2</v>
                      </c>
                      <c r="C12">
                        <v>3</v>
                      </c>
                      <c r="D12">
                        <v>-1</v>
                      </c>
                      <c r="E12">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="13" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A13">
                        <v>-2</v>
                      </c>
                      <c r="B13">
                        <v>2</v>
                      </c>
                      <c r="C13">
                        <v>3</v>
                      </c>
                      <c r="D13">
                        <v>-1</v>
                      </c>
                      <c r="E13">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="14" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A14">
                        <v>-2</v>
                      </c>
                      <c r="B14">
                        <v>2</v>
                      </c>
                      <c r="C14">
                        <v>3</v>
                      </c>
                      <c r="D14">
                        <v>-1</v>
                      </c>
                      <c r="E14">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="15" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A15">
                        <v>-2</v>
                      </c>
                      <c r="B15">
                        <v>2</v>
                      </c>
                      <c r="C15">
                        <v>3</v>
                      </c>
                      <c r="D15">
                        <v>-1</v>
                      </c>
                      <c r="E15">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="16" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A16">
                        <v>-2</v>
                      </c>
                      <c r="B16">
                        <v>2</v>
                      </c>
                      <c r="C16">
                        <v>3</v>
                      </c>
                      <c r="D16">
                        <v>-1</v>
                      </c>
                      <c r="E16">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="17" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A17">
                        <v>-2</v>
                      </c>
                      <c r="B17">
                        <v>2</v>
                      </c>
                      <c r="C17">
                        <v>3</v>
                      </c>
                      <c r="D17">
                        <v>-1</v>
                      </c>
                      <c r="E17">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="18" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A18">
                        <v>-2</v>
                      </c>
                      <c r="B18">
                        <v>2</v>
                      </c>
                      <c r="C18">
                        <v>3</v>
                      </c>
                      <c r="D18">
                        <v>-1</v>
                      </c>
                      <c r="E18">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="19" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A19">
                        <v>-2</v>
                      </c>
                      <c r="B19">
                        <v>2</v>
                      </c>
                      <c r="C19">
                        <v>3</v>
                      </c>
                      <c r="D19">
                        <v>-1</v>
                      </c>
                      <c r="E19">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="20" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A20">
                        <v>-2</v>
                      </c>
                      <c r="B20">
                        <v>2</v>
                      </c>
                      <c r="C20">
                        <v>3</v>
                      </c>
                      <c r="D20">
                        <v>-1</v>
                      </c>
                      <c r="E20">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="21" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A21">
                        <v>-2</v>
                      </c>
                      <c r="B21">
                        <v>2</v>
                      </c>
                      <c r="C21">
                        <v>3</v>
                      </c>
                      <c r="D21">
                        <v>-1</v>
                      </c>
                      <c r="E21">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="22" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A22">
                        <v>-2</v>
                      </c>
                      <c r="B22">
                        <v>2</v>
                      </c>
                      <c r="C22">
                        <v>3</v>
                      </c>
                      <c r="D22">
                        <v>-1</v>
                      </c>
                      <c r="E22">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="23" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A23">
                        <v>-2</v>
                      </c>
                      <c r="B23">
                        <v>2</v>
                      </c>
                      <c r="C23">
                        <v>3</v>
                      </c>
                      <c r="D23">
                        <v>-1</v>
                      </c>
                      <c r="E23">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="24" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A24">
                        <v>-2</v>
                      </c>
                      <c r="B24">
                        <v>2</v>
                      </c>
                      <c r="C24">
                        <v>3</v>
                      </c>
                      <c r="D24">
                        <v>-1</v>
                      </c>
                      <c r="E24">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="25" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A25">
                        <v>-2</v>
                      </c>
                      <c r="B25">
                        <v>2</v>
                      </c>
                      <c r="C25">
                        <v>3</v>
                      </c>
                      <c r="D25">
                        <v>-1</v>
                      </c>
                      <c r="E25">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="26" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A26">
                        <v>-2</v>
                      </c>
                      <c r="B26">
                        <v>2</v>
                      </c>
                      <c r="C26">
                        <v>3</v>
                      </c>
                      <c r="D26">
                        <v>-1</v>
                      </c>
                      <c r="E26">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="27" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A27">
                        <v>-2</v>
                      </c>
                      <c r="B27">
                        <v>2</v>
                      </c>
                      <c r="C27">
                        <v>3</v>
                      </c>
                      <c r="D27">
                        <v>-1</v>
                      </c>
                      <c r="E27">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="28" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A28">
                        <v>-2</v>
                      </c>
                      <c r="B28">
                        <v>2</v>
                      </c>
                      <c r="C28">
                        <v>3</v>
                      </c>
                      <c r="D28">
                        <v>-1</v>
                      </c>
                      <c r="E28">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="29" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A29">
                        <v>-2</v>
                      </c>
                      <c r="B29">
                        <v>2</v>
                      </c>
                      <c r="C29">
                        <v>3</v>
                      </c>
                      <c r="D29">
                        <v>-1</v>
                      </c>
                      <c r="E29">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="30" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A30">
                        <v>-2</v>
                      </c>
                      <c r="B30">
                        <v>2</v>
                      </c>
                      <c r="C30">
                        <v>3</v>
                      </c>
                      <c r="D30">
                        <v>-1</v>
                      </c>
                      <c r="E30">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="31" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A31">
                        <v>-2</v>
                      </c>
                      <c r="B31">
                        <v>2</v>
                      </c>
                      <c r="C31">
                        <v>3</v>
                      </c>
                      <c r="D31">
                        <v>-1</v>
                      </c>
                      <c r="E31">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="32" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A32">
                        <v>-2</v>
                      </c>
                      <c r="B32">
                        <v>2</v>
                      </c>
                      <c r="C32">
                        <v>3</v>
                      </c>
                      <c r="D32">
                        <v>-1</v>
                      </c>
                      <c r="E32">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="33" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A33">
                        <v>-2</v>
                      </c>
                      <c r="B33">
                        <v>2</v>
                      </c>
                      <c r="C33">
                        <v>3</v>
                      </c>
                      <c r="D33">
                        <v>-1</v>
                      </c>
                      <c r="E33">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="34" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A34">
                        <v>-2</v>
                      </c>
                      <c r="B34">
                        <v>2</v>
                      </c>
                      <c r="C34">
                        <v>3</v>
                      </c>
                      <c r="D34">
                        <v>-1</v>
                      </c>
                      <c r="E34">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="35" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A35">
                        <v>-2</v>
                      </c>
                      <c r="B35">
                        <v>2</v>
                      </c>
                      <c r="C35">
                        <v>3</v>
                      </c>
                      <c r="D35">
                        <v>-1</v>
                      </c>
                      <c r="E35">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="36" spans="1:5" x14ac:dyDescent="0.25">
                      <c r="A36">
                        <v>-2</v>
                      </c>
                      <c r="B36">
                        <v>2</v>
                      </c>
                      <c r="C36">
                        <v>3</v>
                      </c>
                      <c r="D36">
                        <v>-1</v>
                      </c>
                      <c r="E36">
                        <v>0</v>
                      </c>
                    </row>
                  </sheetData>
                  <pageMargins left="0.7" right="0.7" top="0.75" bottom="0.75" header="0.3" footer="0.3"/>
                  <extLst>
                    <ext xmlns:x14="http://schemas.microsoft.com/office/spreadsheetml/2009/9/main" uri="{05C60535-1F16-4fd2-B633-F4F36F0B64E0}">
                      <x14:sparklineGroups xmlns:xm="http://schemas.microsoft.com/office/excel/2006/main">
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="1"/>
                          <x14:colorNegative theme="9"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="8"/>
                          <x14:colorFirst theme="4"/>
                          <x14:colorLast theme="5"/>
                          <x14:colorHigh theme="6"/>
                          <x14:colorLow theme="7"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A36:E36</xm:f>
                              <xm:sqref>F36</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="3"/>
                          <x14:colorNegative theme="9"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="8"/>
                          <x14:colorFirst theme="4"/>
                          <x14:colorLast theme="5"/>
                          <x14:colorHigh theme="6"/>
                          <x14:colorLow theme="7"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A35:E35</xm:f>
                              <xm:sqref>F35</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries rgb="FF00B050"/>
                          <x14:colorNegative rgb="FFFF0000"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers rgb="FF0070C0"/>
                          <x14:colorFirst rgb="FFFFC000"/>
                          <x14:colorLast rgb="FFFFC000"/>
                          <x14:colorHigh rgb="FF00B050"/>
                          <x14:colorLow rgb="FFFF0000"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A34:E34</xm:f>
                              <xm:sqref>F34</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries rgb="FFC6EFCE"/>
                          <x14:colorNegative rgb="FFFFC7CE"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers rgb="FF8CADD6"/>
                          <x14:colorFirst rgb="FFFFDC47"/>
                          <x14:colorLast rgb="FFFFEB9C"/>
                          <x14:colorHigh rgb="FF60D276"/>
                          <x14:colorLow rgb="FFFF5367"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A33:E33</xm:f>
                              <xm:sqref>F33</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries rgb="FF5687C2"/>
                          <x14:colorNegative rgb="FFFFB620"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers rgb="FFD70077"/>
                          <x14:colorFirst rgb="FF777777"/>
                          <x14:colorLast rgb="FF359CEB"/>
                          <x14:colorHigh rgb="FF56BE79"/>
                          <x14:colorLow rgb="FFFF5055"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A32:E32</xm:f>
                              <xm:sqref>F32</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries rgb="FF5F5F5F"/>
                          <x14:colorNegative rgb="FFFFB620"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers rgb="FFD70077"/>
                          <x14:colorFirst rgb="FF5687C2"/>
                          <x14:colorLast rgb="FF359CEB"/>
                          <x14:colorHigh rgb="FF56BE79"/>
                          <x14:colorLow rgb="FFFF5055"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A31:E31</xm:f>
                              <xm:sqref>F31</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries rgb="FF0070C0"/>
                          <x14:colorNegative rgb="FF000000"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers rgb="FF000000"/>
                          <x14:colorFirst rgb="FF000000"/>
                          <x14:colorLast rgb="FF000000"/>
                          <x14:colorHigh rgb="FF000000"/>
                          <x14:colorLow rgb="FF000000"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A30:E30</xm:f>
                              <xm:sqref>F30</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries rgb="FF376092"/>
                          <x14:colorNegative rgb="FFD00000"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers rgb="FFD00000"/>
                          <x14:colorFirst rgb="FFD00000"/>
                          <x14:colorLast rgb="FFD00000"/>
                          <x14:colorHigh rgb="FFD00000"/>
                          <x14:colorLow rgb="FFD00000"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A29:E29</xm:f>
                              <xm:sqref>F29</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries rgb="FF000000"/>
                          <x14:colorNegative rgb="FF0070C0"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers rgb="FF0070C0"/>
                          <x14:colorFirst rgb="FF0070C0"/>
                          <x14:colorLast rgb="FF0070C0"/>
                          <x14:colorHigh rgb="FF0070C0"/>
                          <x14:colorLow rgb="FF0070C0"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A28:E28</xm:f>
                              <xm:sqref>F28</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries rgb="FF323232"/>
                          <x14:colorNegative rgb="FFD00000"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers rgb="FFD00000"/>
                          <x14:colorFirst rgb="FFD00000"/>
                          <x14:colorLast rgb="FFD00000"/>
                          <x14:colorHigh rgb="FFD00000"/>
                          <x14:colorLow rgb="FFD00000"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A27:E27</xm:f>
                              <xm:sqref>F27</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="1" tint="0.34998626667073579"/>
                          <x14:colorNegative theme="0" tint="-0.249977111117893"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="0" tint="-0.249977111117893"/>
                          <x14:colorFirst theme="0" tint="-0.249977111117893"/>
                          <x14:colorLast theme="0" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="0" tint="-0.249977111117893"/>
                          <x14:colorLow theme="0" tint="-0.249977111117893"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A26:E26</xm:f>
                              <xm:sqref>F26</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="1" tint="0.499984740745262"/>
                          <x14:colorNegative theme="1" tint="0.249977111117893"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="1" tint="0.249977111117893"/>
                          <x14:colorFirst theme="1" tint="0.249977111117893"/>
                          <x14:colorLast theme="1" tint="0.249977111117893"/>
                          <x14:colorHigh theme="1" tint="0.249977111117893"/>
                          <x14:colorLow theme="1" tint="0.249977111117893"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A25:E25</xm:f>
                              <xm:sqref>F25</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="9" tint="0.39997558519241921"/>
                          <x14:colorNegative theme="0" tint="-0.499984740745262"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="9" tint="0.79998168889431442"/>
                          <x14:colorFirst theme="9" tint="-0.249977111117893"/>
                          <x14:colorLast theme="9" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="9" tint="-0.499984740745262"/>
                          <x14:colorLow theme="9" tint="-0.499984740745262"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A24:E24</xm:f>
                              <xm:sqref>F24</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="8" tint="0.39997558519241921"/>
                          <x14:colorNegative theme="0" tint="-0.499984740745262"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="8" tint="0.79998168889431442"/>
                          <x14:colorFirst theme="8" tint="-0.249977111117893"/>
                          <x14:colorLast theme="8" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="8" tint="-0.499984740745262"/>
                          <x14:colorLow theme="8" tint="-0.499984740745262"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A23:E23</xm:f>
                              <xm:sqref>F23</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="7" tint="0.39997558519241921"/>
                          <x14:colorNegative theme="0" tint="-0.499984740745262"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="7" tint="0.79998168889431442"/>
                          <x14:colorFirst theme="7" tint="-0.249977111117893"/>
                          <x14:colorLast theme="7" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="7" tint="-0.499984740745262"/>
                          <x14:colorLow theme="7" tint="-0.499984740745262"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A22:E22</xm:f>
                              <xm:sqref>F22</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="6" tint="0.39997558519241921"/>
                          <x14:colorNegative theme="0" tint="-0.499984740745262"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="6" tint="0.79998168889431442"/>
                          <x14:colorFirst theme="6" tint="-0.249977111117893"/>
                          <x14:colorLast theme="6" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="6" tint="-0.499984740745262"/>
                          <x14:colorLow theme="6" tint="-0.499984740745262"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A21:E21</xm:f>
                              <xm:sqref>F21</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="5" tint="0.39997558519241921"/>
                          <x14:colorNegative theme="0" tint="-0.499984740745262"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="5" tint="0.79998168889431442"/>
                          <x14:colorFirst theme="5" tint="-0.249977111117893"/>
                          <x14:colorLast theme="5" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="5" tint="-0.499984740745262"/>
                          <x14:colorLow theme="5" tint="-0.499984740745262"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A20:E20</xm:f>
                              <xm:sqref>F20</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="4" tint="0.39997558519241921"/>
                          <x14:colorNegative theme="0" tint="-0.499984740745262"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="4" tint="0.79998168889431442"/>
                          <x14:colorFirst theme="4" tint="-0.249977111117893"/>
                          <x14:colorLast theme="4" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="4" tint="-0.499984740745262"/>
                          <x14:colorLow theme="4" tint="-0.499984740745262"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A19:E19</xm:f>
                              <xm:sqref>F19</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="9"/>
                          <x14:colorNegative theme="4"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="9" tint="-0.249977111117893"/>
                          <x14:colorFirst theme="9" tint="-0.249977111117893"/>
                          <x14:colorLast theme="9" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="9" tint="-0.249977111117893"/>
                          <x14:colorLow theme="9" tint="-0.249977111117893"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A18:E18</xm:f>
                              <xm:sqref>F18</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="8"/>
                          <x14:colorNegative theme="9"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="8" tint="-0.249977111117893"/>
                          <x14:colorFirst theme="8" tint="-0.249977111117893"/>
                          <x14:colorLast theme="8" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="8" tint="-0.249977111117893"/>
                          <x14:colorLow theme="8" tint="-0.249977111117893"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A17:E17</xm:f>
                              <xm:sqref>F17</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="7"/>
                          <x14:colorNegative theme="8"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="7" tint="-0.249977111117893"/>
                          <x14:colorFirst theme="7" tint="-0.249977111117893"/>
                          <x14:colorLast theme="7" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="7" tint="-0.249977111117893"/>
                          <x14:colorLow theme="7" tint="-0.249977111117893"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A16:E16</xm:f>
                              <xm:sqref>F16</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="6"/>
                          <x14:colorNegative theme="7"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="6" tint="-0.249977111117893"/>
                          <x14:colorFirst theme="6" tint="-0.249977111117893"/>
                          <x14:colorLast theme="6" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="6" tint="-0.249977111117893"/>
                          <x14:colorLow theme="6" tint="-0.249977111117893"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A15:E15</xm:f>
                              <xm:sqref>F15</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="5"/>
                          <x14:colorNegative theme="6"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="5" tint="-0.249977111117893"/>
                          <x14:colorFirst theme="5" tint="-0.249977111117893"/>
                          <x14:colorLast theme="5" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="5" tint="-0.249977111117893"/>
                          <x14:colorLow theme="5" tint="-0.249977111117893"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A14:E14</xm:f>
                              <xm:sqref>F14</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="4"/>
                          <x14:colorNegative theme="5"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="4" tint="-0.249977111117893"/>
                          <x14:colorFirst theme="4" tint="-0.249977111117893"/>
                          <x14:colorLast theme="4" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="4" tint="-0.249977111117893"/>
                          <x14:colorLow theme="4" tint="-0.249977111117893"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A13:E13</xm:f>
                              <xm:sqref>F13</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="9" tint="-0.249977111117893"/>
                          <x14:colorNegative theme="4"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="4" tint="-0.249977111117893"/>
                          <x14:colorFirst theme="4" tint="-0.249977111117893"/>
                          <x14:colorLast theme="4" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="4" tint="-0.249977111117893"/>
                          <x14:colorLow theme="4" tint="-0.249977111117893"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A12:E12</xm:f>
                              <xm:sqref>F12</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="8" tint="-0.249977111117893"/>
                          <x14:colorNegative theme="9"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="9" tint="-0.249977111117893"/>
                          <x14:colorFirst theme="9" tint="-0.249977111117893"/>
                          <x14:colorLast theme="9" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="9" tint="-0.249977111117893"/>
                          <x14:colorLow theme="9" tint="-0.249977111117893"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A11:E11</xm:f>
                              <xm:sqref>F11</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="7" tint="-0.249977111117893"/>
                          <x14:colorNegative theme="8"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="8" tint="-0.249977111117893"/>
                          <x14:colorFirst theme="8" tint="-0.249977111117893"/>
                          <x14:colorLast theme="8" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="8" tint="-0.249977111117893"/>
                          <x14:colorLow theme="8" tint="-0.249977111117893"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A10:E10</xm:f>
                              <xm:sqref>F10</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="6" tint="-0.249977111117893"/>
                          <x14:colorNegative theme="7"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="7" tint="-0.249977111117893"/>
                          <x14:colorFirst theme="7" tint="-0.249977111117893"/>
                          <x14:colorLast theme="7" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="7" tint="-0.249977111117893"/>
                          <x14:colorLow theme="7" tint="-0.249977111117893"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A9:E9</xm:f>
                              <xm:sqref>F9</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="5" tint="-0.249977111117893"/>
                          <x14:colorNegative theme="6"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="6" tint="-0.249977111117893"/>
                          <x14:colorFirst theme="6" tint="-0.249977111117893"/>
                          <x14:colorLast theme="6" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="6" tint="-0.249977111117893"/>
                          <x14:colorLow theme="6" tint="-0.249977111117893"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A8:E8</xm:f>
                              <xm:sqref>F8</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="4" tint="-0.249977111117893"/>
                          <x14:colorNegative theme="5"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="5" tint="-0.249977111117893"/>
                          <x14:colorFirst theme="5" tint="-0.249977111117893"/>
                          <x14:colorLast theme="5" tint="-0.249977111117893"/>
                          <x14:colorHigh theme="5" tint="-0.249977111117893"/>
                          <x14:colorLow theme="5" tint="-0.249977111117893"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A7:E7</xm:f>
                              <xm:sqref>F7</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="9" tint="-0.499984740745262"/>
                          <x14:colorNegative theme="4"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="9" tint="-0.499984740745262"/>
                          <x14:colorFirst theme="9" tint="0.39997558519241921"/>
                          <x14:colorLast theme="9" tint="0.39997558519241921"/>
                          <x14:colorHigh theme="9"/>
                          <x14:colorLow theme="9"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A6:E6</xm:f>
                              <xm:sqref>F6</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="8" tint="-0.499984740745262"/>
                          <x14:colorNegative theme="9"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="8" tint="-0.499984740745262"/>
                          <x14:colorFirst theme="8" tint="0.39997558519241921"/>
                          <x14:colorLast theme="8" tint="0.39997558519241921"/>
                          <x14:colorHigh theme="8"/>
                          <x14:colorLow theme="8"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A5:E5</xm:f>
                              <xm:sqref>F5</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="7" tint="-0.499984740745262"/>
                          <x14:colorNegative theme="8"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="7" tint="-0.499984740745262"/>
                          <x14:colorFirst theme="7" tint="0.39997558519241921"/>
                          <x14:colorLast theme="7" tint="0.39997558519241921"/>
                          <x14:colorHigh theme="7"/>
                          <x14:colorLow theme="7"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A4:E4</xm:f>
                              <xm:sqref>F4</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="6" tint="-0.499984740745262"/>
                          <x14:colorNegative theme="7"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="6" tint="-0.499984740745262"/>
                          <x14:colorFirst theme="6" tint="0.39997558519241921"/>
                          <x14:colorLast theme="6" tint="0.39997558519241921"/>
                          <x14:colorHigh theme="6"/>
                          <x14:colorLow theme="6"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A3:E3</xm:f>
                              <xm:sqref>F3</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="5" tint="-0.499984740745262"/>
                          <x14:colorNegative theme="6"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="5" tint="-0.499984740745262"/>
                          <x14:colorFirst theme="5" tint="0.39997558519241921"/>
                          <x14:colorLast theme="5" tint="0.39997558519241921"/>
                          <x14:colorHigh theme="5"/>
                          <x14:colorLow theme="5"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A2:E2</xm:f>
                              <xm:sqref>F2</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                        <x14:sparklineGroup displayEmptyCellsAs="gap">
                          <x14:colorSeries theme="4" tint="-0.499984740745262"/>
                          <x14:colorNegative theme="5"/>
                          <x14:colorAxis rgb="FF000000"/>
                          <x14:colorMarkers theme="4" tint="-0.499984740745262"/>
                          <x14:colorFirst theme="4" tint="0.39997558519241921"/>
                          <x14:colorLast theme="4" tint="0.39997558519241921"/>
                          <x14:colorHigh theme="4"/>
                          <x14:colorLow theme="4"/>
                          <x14:sparklines>
                            <x14:sparkline>
                              <xm:f>Sheet1!A1:E1</xm:f>
                              <xm:sqref>F1</xm:sqref>
                            </x14:sparkline>
                          </x14:sparklines>
                        </x14:sparklineGroup>
                      </x14:sparklineGroups>
                    </ext>
                  </extLst>
                </worksheet>
                """
        )

        got = _xml_to_list(fh.getvalue())

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...drawing import Drawing


class TestWriteXdrrow(unittest.TestCase):
    """
    Test the Drawing _write_row() method.

    """

    def setUp(self):
        self.fh = StringIO()
        self.drawing = Drawing()
        self.drawing._set_filehandle(self.fh)

    def test_write_row(self):
        """Test the _write_row() method"""

        self.drawing._write_row(8)

        exp = """<xdr:row>8</xdr:row>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("escapes01.xlsx")

        self.ignore_files = [
            "xl/calcChain.xml",
            "[Content_Types].xml",
            "xl/_rels/workbook.xml.rels",
        ]

    def test_create_file(self):
        """Test creation of a file with strings that require XML escaping."""

        workbook = Workbook(self.got_filename)

        worksheet = workbook.add_worksheet("5&4")

        worksheet.write_formula(0, 0, "=IF(1>2,0,1)", None, 1)
        worksheet.write_formula(1, 0, """=CONCATENATE("'","<>&")""", None, "'<>&")
        worksheet.write_formula(2, 0, '=1&"b"', None, "1b")
        worksheet.write_formula(3, 0, """="'\"""", None, "'")
        worksheet.write_formula(4, 0, '=""""', None, '"')
        worksheet.write_formula(5, 0, '="&" & "&"', None, "&&")

        worksheet.write_string(7, 0, '"&<>')

        workbook.close()

        self.assertExcelEqual()

    def test_create_file_write(self):
        """Test formulas with write() method."""

        workbook = Workbook(self.got_filename)

        worksheet = workbook.add_worksheet("5&4")

        worksheet.write(0, 0, "=IF(1>2,0,1)", None, 1)
        worksheet.write(1, 0, """=CONCATENATE("'","<>&")""", None, "'<>&")
        worksheet.write(2, 0, '=1&"b"', None, "1b")
        worksheet.write(3, 0, """="'\"""", None, "'")
        worksheet.write(4, 0, '=""""', None, '"')
        worksheet.write(5, 0, '="&" & "&"', None, "&&")

        worksheet.write_string(7, 0, '"&<>')

        workbook.close()

        self.assertExcelEqual()

    def test_create_file_A1(self):
        """Test formulas with A1 notation."""

        workbook = Workbook(self.got_filename)

        worksheet = workbook.add_worksheet("5&4")

        worksheet.write_formula("A1", "=IF(1>2,0,1)", None, 1)
        worksheet.write_formula("A2", """=CONCATENATE("'","<>&")""", None, "'<>&")
        worksheet.write_formula("A3", '=1&"b"', None, "1b")
        worksheet.write_formula("A4", """="'\"""", None, "'")
        worksheet.write_formula("A5", '=""""', None, '"')
        worksheet.write_formula("A6", '="&" & "&"', None, "&&")

        worksheet.write_string(7, 0, '"&<>')

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ..helperfunctions import _xml_to_list
from ...relationships import Relationships


class TestAssembleRelationships(unittest.TestCase):
    """
    Test assembling a complete Relationships file.

    """

    def test_assemble_xml_file(self):
        """Test writing an Relationships file."""
        self.maxDiff = None

        fh = StringIO()
        rels = Relationships()
        rels._set_filehandle(fh)

        rels._add_document_relationship("/worksheet", "worksheets/sheet1.xml")
        rels._add_document_relationship("/theme", "theme/theme1.xml")
        rels._add_document_relationship("/styles", "styles.xml")
        rels._add_document_relationship("/sharedStrings", "sharedStrings.xml")
        rels._add_document_relationship("/calcChain", "calcChain.xml")

        rels._assemble_xml_file()

        exp = _xml_to_list(
            """
                <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
                <Relationships xmlns="http://schemas.openxmlformats.org/package/2006/relationships">
                  <Relationship Id="rId1" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/worksheet" Target="worksheets/sheet1.xml"/>
                  <Relationship Id="rId2" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/theme" Target="theme/theme1.xml"/>
                  <Relationship Id="rId3" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/styles" Target="styles.xml"/>
                  <Relationship Id="rId4" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/sharedStrings" Target="sharedStrings.xml"/>
                  <Relationship Id="rId5" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/calcChain" Target="calcChain.xml"/>
                </Relationships>
                """
        )

        got = _xml_to_list(fh.getvalue())

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("types07.xlsx")

        self.ignore_files = [
            "xl/calcChain.xml",
            "[Content_Types].xml",
            "xl/_rels/workbook.xml.rels",
        ]

    def test_write_nan_and_inf(self):
        """Test writing special numbers."""

        workbook = Workbook(self.got_filename, {"nan_inf_to_errors": True})
        worksheet = workbook.add_worksheet()

        worksheet.write(0, 0, float("nan"))
        worksheet.write(1, 0, float("inf"))
        worksheet.write(2, 0, float("-inf"))

        workbook.close()

        self.assertExcelEqual()

    def test_write_nan_and_inf_write_number(self):
        """Test writing special numbers."""

        workbook = Workbook(self.got_filename, {"nan_inf_to_errors": True})
        worksheet = workbook.add_worksheet()

        worksheet.write_number(0, 0, float("nan"))
        worksheet.write_number(1, 0, float("inf"))
        worksheet.write_number(2, 0, float("-inf"))

        workbook.close()

        self.assertExcelEqual()

    def test_write_nan_and_inf_write_as_string(self):
        """Test writing special numbers."""

        workbook = Workbook(
            self.got_filename, {"nan_inf_to_errors": True, "strings_to_numbers": True}
        )
        worksheet = workbook.add_worksheet()

        worksheet.write(0, 0, "nan")
        worksheet.write(1, 0, "inf")
        worksheet.write(2, 0, "-inf")

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ..helperfunctions import _xml_to_list
from ...drawing import Drawing


class TestAssembleDrawing(unittest.TestCase):
    """
    Test assembling a complete Drawing file.

    """

    def test_assemble_xml_file(self):
        """Test writing a drawing with no cell data."""
        self.maxDiff = None

        fh = StringIO()
        drawing = Drawing()
        drawing._set_filehandle(fh)

        dimensions = [4, 8, 457200, 104775, 12, 22, 152400, 180975, 0, 0]
        drawing_object = drawing._add_drawing_object()
        drawing_object["type"] = 1
        drawing_object["dimensions"] = dimensions
        drawing_object["width"] = 0
        drawing_object["height"] = 0
        drawing_object["description"] = None
        drawing_object["shape"] = None
        drawing_object["anchor"] = 1
        drawing_object["rel_index"] = 1
        drawing_object["url_rel_index"] = 0
        drawing_object["tip"] = None

        drawing.embedded = 1

        drawing._assemble_xml_file()

        exp = _xml_to_list(
            """
                <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
                <xdr:wsDr xmlns:xdr="http://schemas.openxmlformats.org/drawingml/2006/spreadsheetDrawing" xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main">
                  <xdr:twoCellAnchor>
                    <xdr:from>
                      <xdr:col>4</xdr:col>
                      <xdr:colOff>457200</xdr:colOff>
                      <xdr:row>8</xdr:row>
                      <xdr:rowOff>104775</xdr:rowOff>
                    </xdr:from>
                    <xdr:to>
                      <xdr:col>12</xdr:col>
                      <xdr:colOff>152400</xdr:colOff>
                      <xdr:row>22</xdr:row>
                      <xdr:rowOff>180975</xdr:rowOff>
                    </xdr:to>
                    <xdr:graphicFrame macro="">
                      <xdr:nvGraphicFramePr>
                        <xdr:cNvPr id="2" name="Chart 1"/>
                        <xdr:cNvGraphicFramePr/>
                      </xdr:nvGraphicFramePr>
                      <xdr:xfrm>
                        <a:off x="0" y="0"/>
                        <a:ext cx="0" cy="0"/>
                      </xdr:xfrm>
                      <a:graphic>
                        <a:graphicData uri="http://schemas.openxmlformats.org/drawingml/2006/chart">
                          <c:chart xmlns:c="http://schemas.openxmlformats.org/drawingml/2006/chart" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" r:id="rId1"/>
                        </a:graphicData>
                      </a:graphic>
                    </xdr:graphicFrame>
                    <xdr:clientData/>
                  </xdr:twoCellAnchor>
                </xdr:wsDr>
                """
        )

        got = _xml_to_list(fh.getvalue())

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("optimize06.xlsx")

    def test_create_file(self):
        """Test the creation of a simple XlsxWriter file."""

        workbook = Workbook(
            self.got_filename, {"constant_memory": True, "in_memory": False}
        )
        worksheet = workbook.add_worksheet()

        # Test that control characters and any other single byte characters are
        # handled correctly by the SharedStrings module. We skip chr 34 = " in
        # this test since it isn't encoded by Excel as &quot;.
        ordinals = list(range(0, 34))
        ordinals.extend(range(35, 128))

        for i in ordinals:
            worksheet.write_string(i, 0, chr(i))

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ..helperfunctions import _xml_to_list
from ...worksheet import Worksheet
from ...sharedstrings import SharedStringTable


class TestAssembleWorksheet(unittest.TestCase):
    """
    Test assembling a complete Worksheet file.

    """

    def test_assemble_xml_file(self):
        """Test writing a worksheet with strings in cells."""
        self.maxDiff = None

        fh = StringIO()
        worksheet = Worksheet()
        worksheet._set_filehandle(fh)
        worksheet.str_table = SharedStringTable()

        worksheet.select()

        # Write some strings.
        worksheet.write_string(0, 0, "Foo")
        worksheet.write_string(2, 0, "Bar")
        worksheet.write_string(2, 3, "Baz")

        worksheet._assemble_xml_file()

        exp = _xml_to_list(
            """
                <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
                <worksheet xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships">
                  <dimension ref="A1:D3"/>
                  <sheetViews>
                    <sheetView tabSelected="1" workbookViewId="0"/>
                  </sheetViews>
                  <sheetFormatPr defaultRowHeight="15"/>
                  <sheetData>
                    <row r="1" spans="1:4">
                      <c r="A1" t="s">
                        <v>0</v>
                      </c>
                    </row>
                    <row r="3" spans="1:4">
                      <c r="A3" t="s">
                        <v>1</v>
                      </c>
                      <c r="D3" t="s">
                        <v>2</v>
                      </c>
                    </row>
                  </sheetData>
                  <pageMargins left="0.7" right="0.7" top="0.75" bottom="0.75" header="0.3" footer="0.3"/>
                </worksheet>
                """
        )

        got = _xml_to_list(fh.getvalue())

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from datetime import date
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("simple01.xlsx")

    def test_create_file(self):
        """Test the creation of a simple workbook."""

        workbook = Workbook(self.got_filename)
        worksheet = workbook.add_worksheet()

        worksheet.write_string(0, 0, "Hello")
        worksheet.write_number(1, 0, 123)

        workbook.close()

        self.assertExcelEqual()

    def test_create_file_A1(self):
        """Test the creation of a simple workbook with A1 notation."""

        workbook = Workbook(self.got_filename)
        worksheet = workbook.add_worksheet()

        worksheet.write_string("A1", "Hello")
        worksheet.write_number("A2", 123)

        workbook.close()

        self.assertExcelEqual()

    def test_create_file_write(self):
        """Test the creation of a simple workbook using write()."""

        workbook = Workbook(self.got_filename)
        worksheet = workbook.add_worksheet()

        worksheet.write(0, 0, "Hello")
        worksheet.write(1, 0, 123)

        workbook.close()

        self.assertExcelEqual()

    def test_create_file_with_statement(self):
        """Test the creation of a simple workbook using `with` statement."""

        with Workbook(self.got_filename) as workbook:
            worksheet = workbook.add_worksheet()

            worksheet.write(0, 0, "Hello")
            worksheet.write(1, 0, 123)

        self.assertExcelEqual()

    def test_create_file_write_A1(self):
        """Test the creation of a simple workbook using write() with A1."""

        workbook = Workbook(self.got_filename)
        worksheet = workbook.add_worksheet()

        worksheet.write("A1", "Hello")
        worksheet.write("A2", 123)

        workbook.close()

        self.assertExcelEqual()

    def test_create_file_kwargs(self):
        """Test the creation of a simple workbook with keyword args."""

        workbook = Workbook(self.got_filename)
        worksheet = workbook.add_worksheet()

        worksheet.write_string(row=0, col=0, string="Hello")
        worksheet.write_number(row=1, col=0, number=123)

        workbook.close()

        self.assertExcelEqual()

    def test_create_file_write_date_default(self):
        """Test writing a datetime without a format. Issue #33"""

        workbook = Workbook(self.got_filename)
        worksheet = workbook.add_worksheet()

        worksheet.write("A1", "Hello")
        worksheet.write("A2", date(1900, 5, 2))

        workbook.close()

        self.assertExcelEqual()

    def test_create_file_in_memory(self):
        """Test the creation of a simple workbook."""

        workbook = Workbook(self.got_filename, {"in_memory": True})
        worksheet = workbook.add_worksheet()

        worksheet.write_string(0, 0, "Hello")
        worksheet.write_number(1, 0, 123)

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ..helperfunctions import _xml_to_list
from ...worksheet import Worksheet


class TestAssembleWorksheet(unittest.TestCase):
    """
    Test assembling a complete Worksheet file.

    """

    def test_assemble_xml_file(self):
        """Test writing a worksheet with conditional formatting."""
        self.maxDiff = None

        fh = StringIO()
        worksheet = Worksheet()
        worksheet._set_filehandle(fh)
        worksheet.select()
        worksheet.index = 0

        worksheet.conditional_format(
            "A1",
            {
                "type": "data_bar",
                "bar_negative_color_same": True,
            },
        )

        worksheet.conditional_format(
            "A2:B2",
            {
                "type": "data_bar",
                "bar_color": "#63C384",
                "bar_negative_border_color": "#92D050",
            },
        )

        worksheet.conditional_format(
            "A3:C3",
            {
                "type": "data_bar",
                "bar_color": "#FF555A",
                "bar_negative_border_color_same": True,
            },
        )

        worksheet._assemble_xml_file()

        exp = _xml_to_list(
            """
                <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
                <worksheet xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:mc="http://schemas.openxmlformats.org/markup-compatibility/2006" xmlns:x14ac="http://schemas.microsoft.com/office/spreadsheetml/2009/9/ac" mc:Ignorable="x14ac">
                  <dimension ref="A1"/>
                  <sheetViews>
                    <sheetView tabSelected="1" workbookViewId="0"/>
                  </sheetViews>
                  <sheetFormatPr defaultRowHeight="15" x14ac:dyDescent="0.25"/>
                  <sheetData/>
                  <conditionalFormatting sqref="A1">
                    <cfRule type="dataBar" priority="1">
                      <dataBar>
                        <cfvo type="min"/>
                        <cfvo type="max"/>
                        <color rgb="FF638EC6"/>
                      </dataBar>
                      <extLst>
                        <ext xmlns:x14="http://schemas.microsoft.com/office/spreadsheetml/2009/9/main" uri="{B025F937-C7B1-47D3-B67F-A62EFF666E3E}">
                          <x14:id>{DA7ABA51-AAAA-BBBB-0001-000000000001}</x14:id>
                        </ext>
                      </extLst>
                    </cfRule>
                  </conditionalFormatting>
                  <conditionalFormatting sqref="A2:B2">
                    <cfRule type="dataBar" priority="2">
                      <dataBar>
                        <cfvo type="min"/>
                        <cfvo type="max"/>
                        <color rgb="FF63C384"/>
                      </dataBar>
                      <extLst>
                        <ext xmlns:x14="http://schemas.microsoft.com/office/spreadsheetml/2009/9/main" uri="{B025F937-C7B1-47D3-B67F-A62EFF666E3E}">
                          <x14:id>{DA7ABA51-AAAA-BBBB-0001-000000000002}</x14:id>
                        </ext>
                      </extLst>
                    </cfRule>
                  </conditionalFormatting>
                  <conditionalFormatting sqref="A3:C3">
                    <cfRule type="dataBar" priority="3">
                      <dataBar>
                        <cfvo type="min"/>
                        <cfvo type="max"/>
                        <color rgb="FFFF555A"/>
                      </dataBar>
                      <extLst>
                        <ext xmlns:x14="http://schemas.microsoft.com/office/spreadsheetml/2009/9/main" uri="{B025F937-C7B1-47D3-B67F-A62EFF666E3E}">
                          <x14:id>{DA7ABA51-AAAA-BBBB-0001-000000000003}</x14:id>
                        </ext>
                      </extLst>
                    </cfRule>
                  </conditionalFormatting>
                  <pageMargins left="0.7" right="0.7" top="0.75" bottom="0.75" header="0.3" footer="0.3"/>
                  <extLst>
                    <ext xmlns:x14="http://schemas.microsoft.com/office/spreadsheetml/2009/9/main" uri="{78C0D931-6437-407d-A8EE-F0AAD7539E65}">
                      <x14:conditionalFormattings>
                        <x14:conditionalFormatting xmlns:xm="http://schemas.microsoft.com/office/excel/2006/main">
                          <x14:cfRule type="dataBar" id="{DA7ABA51-AAAA-BBBB-0001-000000000001}">
                            <x14:dataBar minLength="0" maxLength="100" border="1" negativeBarColorSameAsPositive="1" negativeBarBorderColorSameAsPositive="0">
                              <x14:cfvo type="autoMin"/>
                              <x14:cfvo type="autoMax"/>
                              <x14:borderColor rgb="FF638EC6"/>
                              <x14:negativeBorderColor rgb="FFFF0000"/>
                              <x14:axisColor rgb="FF000000"/>
                            </x14:dataBar>
                          </x14:cfRule>
                          <xm:sqref>A1</xm:sqref>
                        </x14:conditionalFormatting>
                        <x14:conditionalFormatting xmlns:xm="http://schemas.microsoft.com/office/excel/2006/main">
                          <x14:cfRule type="dataBar" id="{DA7ABA51-AAAA-BBBB-0001-000000000002}">
                            <x14:dataBar minLength="0" maxLength="100" border="1" negativeBarBorderColorSameAsPositive="0">
                              <x14:cfvo type="autoMin"/>
                              <x14:cfvo type="autoMax"/>
                              <x14:borderColor rgb="FF63C384"/>
                              <x14:negativeFillColor rgb="FFFF0000"/>
                              <x14:negativeBorderColor rgb="FF92D050"/>
                              <x14:axisColor rgb="FF000000"/>
                            </x14:dataBar>
                          </x14:cfRule>
                          <xm:sqref>A2:B2</xm:sqref>
                        </x14:conditionalFormatting>
                        <x14:conditionalFormatting xmlns:xm="http://schemas.microsoft.com/office/excel/2006/main">
                          <x14:cfRule type="dataBar" id="{DA7ABA51-AAAA-BBBB-0001-000000000003}">
                            <x14:dataBar minLength="0" maxLength="100" border="1">
                              <x14:cfvo type="autoMin"/>
                              <x14:cfvo type="autoMax"/>
                              <x14:borderColor rgb="FFFF555A"/>
                              <x14:negativeFillColor rgb="FFFF0000"/>
                              <x14:axisColor rgb="FF000000"/>
                            </x14:dataBar>
                          </x14:cfRule>
                          <xm:sqref>A3:C3</xm:sqref>
                        </x14:conditionalFormatting>
                      </x14:conditionalFormattings>
                    </ext>
                  </extLst>
                </worksheet>
                """
        )

        got = _xml_to_list(fh.getvalue())

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from datetime import date
from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("chart_date04.xlsx")

        self.ignore_elements = {"xl/charts/chart1.xml": ["<c:formatCode"]}

    def test_create_file(self):
        """Test the creation of a simple XlsxWriter file."""

        workbook = Workbook(self.got_filename)

        worksheet = workbook.add_worksheet()
        chart = workbook.add_chart({"type": "line"})
        date_format = workbook.add_format({"num_format": 14})

        chart.axis_ids = [51761152, 51762688]

        worksheet.set_column("A:A", 12)

        dates = [
            date(2013, 1, 1),
            date(2013, 1, 2),
            date(2013, 1, 3),
            date(2013, 1, 4),
            date(2013, 1, 5),
            date(2013, 1, 6),
            date(2013, 1, 7),
            date(2013, 1, 8),
            date(2013, 1, 9),
            date(2013, 1, 10),
        ]

        values = [10, 30, 20, 40, 20, 60, 50, 40, 30, 30]

        worksheet.write_column("A1", dates, date_format)
        worksheet.write_column("B1", values)

        chart.add_series(
            {
                "categories": "=Sheet1!$A$1:$A$10",
                "values": "=Sheet1!$B$1:$B$10",
            }
        )

        chart.set_x_axis(
            {
                "date_axis": True,
                "minor_unit": 1,
                "major_unit": 1,
                "minor_unit_type": "months",
                "major_unit_type": "years",
                "num_format": "dd/mm/yyyy",
                "num_format_linked": True,
            }
        )

        worksheet.insert_chart("E9", chart)

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...comments import Comments


class TestWriteText(unittest.TestCase):
    """
    Test the Comments _write_text_t() method.

    """

    def setUp(self):
        self.fh = StringIO()
        self.comments = Comments()
        self.comments._set_filehandle(self.fh)

    def test_write_text_t_1(self):
        """Test the _write_text_t() method"""

        self.comments._write_text_t("Some text")

        exp = """<t>Some text</t>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_text_t_2(self):
        """Test the _write_text_t() method"""

        self.comments._write_text_t(" Some text")

        exp = """<t xml:space="preserve"> Some text</t>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_text_t_3(self):
        """Test the _write_text_t() method"""

        self.comments._write_text_t("Some text ")

        exp = """<t xml:space="preserve">Some text </t>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_text_t_4(self):
        """Test the _write_text_t() method"""

        self.comments._write_text_t(" Some text ")

        exp = """<t xml:space="preserve"> Some text </t>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_text_t_5(self):
        """Test the _write_text_t() method"""

        self.comments._write_text_t("Some text\n")

        exp = """<t xml:space="preserve">Some text\n</t>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...workbook import Workbook
from ...exceptions import DuplicateTableName


class TestAddTable(unittest.TestCase):
    """
    Test exceptions with add_table().

    """

    def test_duplicate_table_name(self):
        """Test that adding 2 tables with the same name raises an exception."""

        fh = StringIO()
        workbook = Workbook()
        workbook._set_filehandle(fh)

        worksheet = workbook.add_worksheet()

        worksheet.add_table("B1:F3", {"name": "SalesData"})
        worksheet.add_table("B4:F7", {"name": "SalesData"})

        self.assertRaises(DuplicateTableName, workbook._prepare_tables)

        workbook.fileclosed = True



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...workbook import Workbook
from ...exceptions import UndefinedImageSize
from ...exceptions import UnsupportedImageFormat


class TestInsertImage(unittest.TestCase):
    """
    Test exceptions with insert_image().

    """

    def test_undefined_image_size(self):
        """Test adding an image with no height/width data."""

        fh = StringIO()
        workbook = Workbook()
        workbook._set_filehandle(fh)

        worksheet = workbook.add_worksheet()

        worksheet.insert_image("B13", "xlsxwriter/test/comparison/images/nosize.png")

        self.assertRaises(UndefinedImageSize, workbook._prepare_drawings)

        workbook.fileclosed = True

    def test_unsupported_image(self):
        """Test adding an unsupported image type."""

        fh = StringIO()
        workbook = Workbook()
        workbook._set_filehandle(fh)

        worksheet = workbook.add_worksheet()

        worksheet.insert_image(
            "B13", "xlsxwriter/test/comparison/images/unsupported.txt"
        )

        self.assertRaises(UnsupportedImageFormat, workbook._prepare_drawings)

        workbook.fileclosed = True



##############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("format15.xlsx")

    def test_create_file_zero_number_format(self):
        """Test the creation of a simple XlsxWriter file 0 number format."""

        workbook = Workbook(self.got_filename)

        worksheet = workbook.add_worksheet()

        format1 = workbook.add_format({"bold": 1})
        format2 = workbook.add_format({"bold": 1, "num_format": 0})

        worksheet.write("A1", 1, format1)
        worksheet.write("A2", 2, format2)

        workbook.close()

        self.assertExcelEqual()

    def test_create_file_zero_number_format_string(self):
        """Test the creation of a simple XlsxWriter file 0 number format."""

        workbook = Workbook(self.got_filename)

        worksheet = workbook.add_worksheet()

        format1 = workbook.add_format({"bold": 1})
        format2 = workbook.add_format({"bold": 1, "num_format": "0"})

        worksheet.write("A1", 1, format1)
        worksheet.write("A2", 2, format2)

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("chart_combined08.xlsx")

        self.ignore_elements = {
            "xl/charts/chart1.xml": [
                "<c:dispBlanksAs",
                "<c:crossBetween",
                "<c:tickLblPos",
                "<c:auto",
                "<c:valAx>",
                "<c:catAx>",
                "</c:valAx>",
                "</c:catAx>",
                "<c:crosses",
                "<c:lblOffset",
                "<c:lblAlgn",
            ]
        }

    def test_create_file(self):
        """Test the creation of a simple XlsxWriter file."""

        workbook = Workbook(self.got_filename)

        worksheet = workbook.add_worksheet()
        chart1 = workbook.add_chart({"type": "column"})
        chart2 = workbook.add_chart({"type": "scatter"})

        chart1.axis_ids = [81267328, 81297792]
        chart2.axis_ids = [81267328, 81297792]
        chart2.axis2_ids = [89510656, 84556032]

        data = [
            [2, 3, 4, 5, 6],
            [20, 25, 10, 10, 20],
            [5, 10, 15, 10, 5],
        ]

        worksheet.write_column("A1", data[0])
        worksheet.write_column("B1", data[1])
        worksheet.write_column("C1", data[2])

        chart1.add_series(
            {"categories": "=Sheet1!$A$1:$A$5", "values": "=Sheet1!$B$1:$B$5"}
        )

        chart2.add_series(
            {
                "categories": "=Sheet1!$A$1:$A$5",
                "values": "=Sheet1!$C$1:$C$5",
                "y2_axis": 1,
            }
        )

        chart1.combine(chart2)

        worksheet.insert_chart("E9", chart1)

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("chart_pattern05.xlsx")

    def test_create_file(self):
        """Test the creation of a simple XlsxWriter file."""

        workbook = Workbook(self.got_filename)

        worksheet = workbook.add_worksheet()
        chart = workbook.add_chart({"type": "column"})

        chart.axis_ids = [110902272, 110756608]

        data = [
            [2, 2, 2],
            [2, 2, 2],
            [2, 2, 2],
            [2, 2, 2],
            [2, 2, 2],
            [2, 2, 2],
            [2, 2, 2],
            [2, 2, 2],
        ]

        worksheet.write_column("A1", data[0])
        worksheet.write_column("B1", data[1])
        worksheet.write_column("C1", data[2])
        worksheet.write_column("D1", data[3])
        worksheet.write_column("E1", data[4])
        worksheet.write_column("F1", data[5])
        worksheet.write_column("G1", data[6])
        worksheet.write_column("H1", data[7])

        chart.add_series(
            {
                "values": "=Sheet1!$A$1:$A$3",
                "pattern": {
                    "pattern": "percent_25",
                    "fg_color": "#C00000",
                    "bg_color": "#FFFFFF",
                },
            }
        )

        chart.add_series(
            {
                "values": "=Sheet1!$B$1:$B$3",
                "pattern": {
                    "pattern": "percent_75",
                    "fg_color": "#FF0000",
                },
            }
        )

        chart.add_series(
            {
                "values": "=Sheet1!$C$1:$C$3",
                "pattern": {
                    "pattern": "dark_upward_diagonal",
                    "fg_color": "#FFC000",
                },
            }
        )

        chart.add_series(
            {
                "values": "=Sheet1!$D$1:$D$3",
                "pattern": {
                    "pattern": "narrow_horizontal",
                    "fg_color": "#FFFF00",
                },
            }
        )

        chart.add_series(
            {
                "values": "=Sheet1!$E$1:$E$3",
                "pattern": {
                    "pattern": "dashed_vertical",
                    "fg_color": "#92D050",
                },
            }
        )

        chart.add_series(
            {
                "values": "=Sheet1!$F$1:$F$3",
                "pattern": {
                    "pattern": "horizontal_brick",
                    "fg_color": "#00B050",
                },
            }
        )

        chart.add_series(
            {
                "values": "=Sheet1!$G$1:$G$3",
                "pattern": {
                    "pattern": "shingle",
                    "fg_color": "#00B0F0",
                },
            }
        )

        chart.add_series(
            {
                "values": "=Sheet1!$H$1:$H$3",
                "pattern": {
                    "pattern": "large_check",
                    "fg_color": "#0070C0",
                },
            }
        )

        worksheet.insert_chart("E9", chart)

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("set_column01.xlsx")

    def test_create_file(self):
        """Test the creation of a simple XlsxWriter file."""

        workbook = Workbook(self.got_filename)

        worksheet = workbook.add_worksheet()

        worksheet.set_column("A:Z", 0.08)  # Test that the cols are overridden.
        worksheet.set_column("B:B", 0.17)
        worksheet.set_column("C:C", 0.25)
        worksheet.set_column("D:D", 0.33)
        worksheet.set_column("E:E", 0.42)
        worksheet.set_column("F:F", 0.5)
        worksheet.set_column("G:G", 0.58)
        worksheet.set_column("H:H", 0.67)
        worksheet.set_column("I:I", 0.75)
        worksheet.set_column("J:J", 0.83)
        worksheet.set_column("K:K", 0.92)
        worksheet.set_column("L:L", 1)
        worksheet.set_column("M:M", 1.14)
        worksheet.set_column("N:N", 1.29)
        worksheet.set_column("O:O", 1.43)
        worksheet.set_column("P:P", 1.57)
        worksheet.set_column("Q:Q", 1.71)
        worksheet.set_column("R:R", 1.86)
        worksheet.set_column("S:S", 2)
        worksheet.set_column("T:T", 2.14)
        worksheet.set_column("U:U", 2.29)
        worksheet.set_column("V:V", 2.43)
        worksheet.set_column("W:W", 2.57)
        worksheet.set_column("X:X", 2.71)
        worksheet.set_column("Y:Y", 2.86)
        worksheet.set_column("Z:Z", 3)
        worksheet.set_column("AB:AB", 8.57)
        worksheet.set_column("AC:AC", 8.71)
        worksheet.set_column("AD:AD", 8.86)
        worksheet.set_column("AE:AE", 9)
        worksheet.set_column("AF:AF", 9.14)
        worksheet.set_column("AG:AG", 9.29)

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("simple07.xlsx")

    def test_write_nan(self):
        """Test write with NAN/INF. Issue #30"""

        workbook = Workbook(self.got_filename)
        worksheet = workbook.add_worksheet()

        worksheet.write_string(0, 0, "Foo")
        worksheet.write_number(1, 0, 123)
        worksheet.write_string(2, 0, "NAN")
        worksheet.write_string(3, 0, "nan")
        worksheet.write_string(4, 0, "INF")
        worksheet.write_string(5, 0, "infinity")

        workbook.close()

        self.assertExcelEqual()

    def test_create_file_in_memory(self):
        """Test write with NAN/INF. Issue #30"""

        workbook = Workbook(self.got_filename, {"in_memory": True})
        worksheet = workbook.add_worksheet()

        worksheet.write_string(0, 0, "Foo")
        worksheet.write_number(1, 0, 123)
        worksheet.write_string(2, 0, "NAN")
        worksheet.write_string(3, 0, "nan")
        worksheet.write_string(4, 0, "INF")
        worksheet.write_string(5, 0, "infinity")

        workbook.close()

        self.assertExcelEqual()



##############################################################################
#
# Simple Python program to benchmark several Python Excel writing modules.
#
# python bench_excel_writers.py [num_rows] [num_cols]
#
# Copyright 2013-2023, John McNamara, jmcnamara@cpan.org
#
import os
import sys
try:
    from time import process_time
except ImportError:
    from time import clock as process_time

import openpyxl
import xlsxwriter

# Default to 1 sheet with 1000 rows x 50 cols
row_max = 1000
col_max = 50
sheets = 1

if len(sys.argv) > 1:
    row_max = int(sys.argv[1])
    sheets = int(sys.argv[3])

elif len(sys.argv) > 2:
    col_max = int(sys.argv[2])

elif len(sys.argv) > 3:
    sheets = int(sys.argv[3])


def print_elapsed_time(module_name, elapsed, optimised=False):
    """ Print module run times in a consistent format. """
    if optimised:
        module_name += " (optimised)"
    print("    %-22s: %6.2f" % (module_name, elapsed))


def time_xlsxwriter(optimised=False):
    """ Run XlsxWriter in optimised/constant memory mode. """
    options = {}
    module_name = "xlsxwriter"
    if optimised:
        options['constant_memory'] = True

    start_time = process_time()
    filename = 'xlsxwriter_opt.xlsx'

    workbook = xlsxwriter.Workbook(filename,
                                   options=options)
    for r in range(sheets):
        worksheet = workbook.add_worksheet()

        for row in range(0, row_max, 2):
            string_data = ["Row: %d Col: %d" % (row, col) for col in range(col_max)]
            worksheet.write_row(row, 0, string_data)

            num_data = [row + col for col in range(col_max)]
            worksheet.write_row(row + 1, 0, num_data)

    workbook.close()

    elapsed = process_time() - start_time
    print_elapsed_time(module_name, elapsed, optimised)
    os.remove(filename)


def time_openpyxl(optimised=False):
    """ Run OpenPyXL in default mode. """
    module_name = "openpyxl"

    start_time = process_time()
    filename = 'openpyxl.xlsx'

    workbook = openpyxl.Workbook(write_only=optimised)
    for r in range(sheets):
        worksheet = workbook.create_sheet()

        for row in range(row_max // 2):

            string_data = ("Row: %d Col: %d" % (row, col) for col in range(col_max))
            worksheet.append(string_data)

            num_data = (row + col for col in range(col_max))
            worksheet.append(num_data)

    workbook.save(filename)

    elapsed = process_time() - start_time
    print_elapsed_time(module_name, elapsed, optimised)
    os.remove(filename)


print("")
print("Versions:")
print("%s: %s" % ('python', str(sys.version).split()[0]))
print("%s: %s" % ('openpyxl', openpyxl.__version__))
print("%s: %s" % ('xlsxwriter', xlsxwriter.__version__))
print("")

print("Dimensions:")
print("    Rows = %d" % row_max)
print("    Cols = %d" % col_max)
print("    Sheets = %d" % sheets)
print("")

print("Times:")
time_xlsxwriter()
time_xlsxwriter(optimised=True)
time_openpyxl()
time_openpyxl(optimised=True)
print("")




###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("panes01.xlsx")

    def test_create_file(self):
        """Test an XlsxWriter file with panes.."""

        workbook = Workbook(self.got_filename)

        worksheet01 = workbook.add_worksheet()
        worksheet02 = workbook.add_worksheet()
        worksheet03 = workbook.add_worksheet()
        worksheet04 = workbook.add_worksheet()
        worksheet05 = workbook.add_worksheet()
        worksheet06 = workbook.add_worksheet()
        worksheet07 = workbook.add_worksheet()
        worksheet08 = workbook.add_worksheet()
        worksheet09 = workbook.add_worksheet()
        worksheet10 = workbook.add_worksheet()
        worksheet11 = workbook.add_worksheet()
        worksheet12 = workbook.add_worksheet()
        worksheet13 = workbook.add_worksheet()

        worksheet01.write("A1", "Foo")
        worksheet02.write("A1", "Foo")
        worksheet03.write("A1", "Foo")
        worksheet04.write("A1", "Foo")
        worksheet05.write("A1", "Foo")
        worksheet06.write("A1", "Foo")
        worksheet07.write("A1", "Foo")
        worksheet08.write("A1", "Foo")
        worksheet09.write("A1", "Foo")
        worksheet10.write("A1", "Foo")
        worksheet11.write("A1", "Foo")
        worksheet12.write("A1", "Foo")
        worksheet13.write("A1", "Foo")

        worksheet01.freeze_panes("A2")
        worksheet02.freeze_panes("A3")
        worksheet03.freeze_panes("B1")
        worksheet04.freeze_panes("C1")
        worksheet05.freeze_panes("B2")
        worksheet06.freeze_panes("G4")
        worksheet07.freeze_panes(3, 6, 3, 6, 1)
        worksheet08.split_panes(15, 0)
        worksheet09.split_panes(30, 0)
        worksheet10.split_panes(0, 8.46)
        worksheet11.split_panes(0, 17.57)
        worksheet12.split_panes(15, 8.46)
        worksheet13.split_panes(45, 54.14)

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...vml import Vml


class TestWriteVpath(unittest.TestCase):
    """
    Test the Vml _write_path() method.

    """

    def setUp(self):
        self.fh = StringIO()
        self.vml = Vml()
        self.vml._set_filehandle(self.fh)

    def test_write_comment_path_1(self):
        """Test the _write_comment_path() method"""

        self.vml._write_comment_path("t", "rect")

        exp = """<v:path gradientshapeok="t" o:connecttype="rect"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_comment_path_2(self):
        """Test the _write_comment_path() method"""

        self.vml._write_comment_path(None, "none")

        exp = """<v:path o:connecttype="none"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_button_path(self):
        """Test the _write_button_path() method"""

        self.vml._write_button_path()

        exp = """<v:path shadowok="f" o:extrusionok="f" strokeok="f" fillok="f" o:connecttype="rect"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...worksheet import Worksheet


class TestWriteColBreaks(unittest.TestCase):
    """
    Test the Worksheet _write_col_breaks() method.

    """

    def setUp(self):
        self.fh = StringIO()
        self.worksheet = Worksheet()
        self.worksheet._set_filehandle(self.fh)

    def test_write_col_breaks_1(self):
        """Test the _write_col_breaks() method"""

        self.worksheet.vbreaks = [1]

        self.worksheet._write_col_breaks()

        exp = """<colBreaks count="1" manualBreakCount="1"><brk id="1" max="1048575" man="1"/></colBreaks>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_col_breaks_2(self):
        """Test the _write_col_breaks() method"""

        self.worksheet.vbreaks = [8, 3, 1, 0]

        self.worksheet._write_col_breaks()

        exp = """<colBreaks count="3" manualBreakCount="3"><brk id="1" max="1048575" man="1"/><brk id="3" max="1048575" man="1"/><brk id="8" max="1048575" man="1"/></colBreaks>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...worksheet import Worksheet


class TestWriteFilters(unittest.TestCase):
    """
    Test the Worksheet _write_filters() method.

    """

    def setUp(self):
        self.fh = StringIO()
        self.worksheet = Worksheet()
        self.worksheet._set_filehandle(self.fh)

    def test_write_filters_1(self):
        """Test the _write_filters() method"""

        self.worksheet._write_filters(["East"])

        exp = """<filters><filter val="East"/></filters>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_filters_2(self):
        """Test the _write_filters() method"""

        self.worksheet._write_filters(["East", "South"])

        exp = """<filters><filter val="East"/><filter val="South"/></filters>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_filters_3(self):
        """Test the _write_filters() method"""

        self.worksheet._write_filters(["blanks"])

        exp = """<filters blank="1"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from io import StringIO
from ...worksheet import Worksheet
from ...format import Format


class TestWriteRow(unittest.TestCase):
    """
    Test the Worksheet _write_row() method.

    """

    def setUp(self):
        self.fh = StringIO()
        self.worksheet = Worksheet()
        self.worksheet._set_filehandle(self.fh)

    def test_write_row_1(self):
        """Test the _write_row() method"""

        self.worksheet._write_row(0, None)

        exp = """<row r="1">"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_row_2(self):
        """Test the _write_row() method"""

        self.worksheet._write_row(2, "2:2")

        exp = """<row r="3" spans="2:2">"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_row_3(self):
        """Test the _write_row() method"""

        self.worksheet._write_row(1, None, [30, None, 0, 0, 0])

        exp = """<row r="2" ht="30" customHeight="1">"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_row_4(self):
        """Test the _write_row() method"""

        self.worksheet._write_row(3, None, [15, None, 1, 0, 0])

        exp = """<row r="4" hidden="1">"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_row_5(self):
        """Test the _write_row() method"""

        cell_format = Format({"xf_index": 1})

        self.worksheet._write_row(6, None, [15, cell_format, 0, 0, 0])

        exp = """<row r="7" s="1" customFormat="1">"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_row_6(self):
        """Test the _write_row() method"""

        self.worksheet._write_row(9, None, [3, None, 0, 0, 0])

        exp = """<row r="10" ht="3" customHeight="1">"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_row_7(self):
        """Test the _write_row() method"""

        self.worksheet._write_row(12, None, [24, None, 1, 0, 0])

        exp = """<row r="13" ht="24" hidden="1" customHeight="1">"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_row_8(self):
        """Test the _write_row() method"""

        self.worksheet._write_row(12, None, [24, None, 1, 0, 0], 1)

        exp = """<row r="13" ht="24" hidden="1" customHeight="1"/>"""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from datetime import datetime
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("default_date_format01.xlsx")

    def test_create_file_user_date_format(self):
        """Test write_datetime with explicit date format."""

        workbook = Workbook(self.got_filename)

        worksheet = workbook.add_worksheet()

        worksheet.set_column(0, 0, 12)

        format1 = workbook.add_format({"num_format": "yyyy\\-mm\\-dd"})

        date1 = datetime.strptime("2013-07-25", "%Y-%m-%d")

        worksheet.write_datetime(0, 0, date1, format1)

        workbook.close()

        self.assertExcelEqual()

    def test_create_file_default_date_format(self):
        """Test write_datetime with default date format."""

        workbook = Workbook(
            self.got_filename, {"default_date_format": "yyyy\\-mm\\-dd"}
        )

        worksheet = workbook.add_worksheet()

        worksheet.set_column(0, 0, 12)

        date1 = datetime.strptime("2013-07-25", "%Y-%m-%d")

        worksheet.write_datetime(0, 0, date1)

        workbook.close()

        self.assertExcelEqual()

    def test_create_file_default_date_format_write(self):
        """Test write_datetime with default date format."""

        workbook = Workbook(
            self.got_filename, {"default_date_format": "yyyy\\-mm\\-dd"}
        )

        worksheet = workbook.add_worksheet()

        worksheet.set_column(0, 0, 12)

        date1 = datetime.strptime("2013-07-25", "%Y-%m-%d")

        worksheet.write("A1", date1)

        workbook.close()

        self.assertExcelEqual()

    def test_create_file_default_date_format_write_row(self):
        """Test write_row with default date format."""

        workbook = Workbook(
            self.got_filename, {"default_date_format": "yyyy\\-mm\\-dd"}
        )

        worksheet = workbook.add_worksheet()

        worksheet.set_column(0, 0, 12)

        date1 = datetime.strptime("2013-07-25", "%Y-%m-%d")

        worksheet.write_row("A1", [date1])

        workbook.close()

        self.assertExcelEqual()

    def test_create_file_default_date_format_write_column(self):
        """Test write_column with default date format."""

        workbook = Workbook(
            self.got_filename, {"default_date_format": "yyyy\\-mm\\-dd"}
        )

        worksheet = workbook.add_worksheet()

        worksheet.set_column(0, 0, 12)

        date1 = datetime.strptime("2013-07-25", "%Y-%m-%d")

        worksheet.write_column(0, 0, [date1])

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

from ..excel_comparison_test import ExcelComparisonTest
from datetime import datetime
from ...workbook import Workbook


class TestCompareXLSXFiles(ExcelComparisonTest):
    """
    Test file created by XlsxWriter against a file created by Excel.

    """

    def setUp(self):
        self.set_filename("tutorial03.xlsx")

        self.ignore_files = [
            "xl/calcChain.xml",
            "[Content_Types].xml",
            "xl/_rels/workbook.xml.rels",
        ]

    def test_create_file(self):
        """Example spreadsheet used in the tutorial."""

        workbook = Workbook(self.got_filename)
        worksheet = workbook.add_worksheet()

        # Add a bold format to use to highlight cells.
        bold = workbook.add_format({"bold": 1})

        # Add a number format for cells with money.
        money_format = workbook.add_format({"num_format": "\\$#,##0"})

        # Add an Excel date format.
        date_format = workbook.add_format({"num_format": "mmmm\\ d\\ yyyy"})

        # Adjust the column width.
        worksheet.set_column("B:B", 15)

        # Write some data headers.
        worksheet.write("A1", "Item", bold)
        worksheet.write("B1", "Date", bold)
        worksheet.write("C1", "Cost", bold)

        # Some data we want to write to the worksheet.
        expenses = (
            ["Rent", "2013-01-13", 1000],
            ["Gas", "2013-01-14", 100],
            ["Food", "2013-01-16", 300],
            ["Gym", "2013-01-20", 50],
        )

        # Start from the first cell below the headers.
        row = 1
        col = 0

        for item, date_str, cost in expenses:
            # Convert the date string into a datetime object.
            date = datetime.strptime(date_str, "%Y-%m-%d")

            worksheet.write_string(row, col, item)
            worksheet.write_datetime(row, col + 1, date, date_format)
            worksheet.write_number(row, col + 2, cost, money_format)
            row += 1

        # Write a total using a formula.
        worksheet.write(row, 0, "Total", bold)
        worksheet.write(row, 2, "=SUM(C2:C5)", money_format, 1450)

        workbook.close()

        self.assertExcelEqual()

    def test_create_file2(self):
        """
        Example spreadsheet used in the tutorial. Format creation is
        re-ordered to ensure correct internal order is maintained.

        """

        workbook = Workbook(self.got_filename)
        worksheet = workbook.add_worksheet()

        # Same as above but re-ordered.
        date_format = workbook.add_format({"num_format": "mmmm\\ d\\ yyyy"})
        money_format = workbook.add_format({"num_format": "\\$#,##0"})
        bold = workbook.add_format({"bold": 1})

        # Adjust the column width.
        worksheet.set_column(1, 1, 15)

        # Write some data headers.
        worksheet.write("A1", "Item", bold)
        worksheet.write("B1", "Date", bold)
        worksheet.write("C1", "Cost", bold)

        # Some data we want to write to the worksheet.
        expenses = (
            ["Rent", "2013-01-13", 1000],
            ["Gas", "2013-01-14", 100],
            ["Food", "2013-01-16", 300],
            ["Gym", "2013-01-20", 50],
        )

        # Start from the first cell below the headers.
        row = 1
        col = 0

        for item, date_str, cost in expenses:
            # Convert the date string into a datetime object.
            date = datetime.strptime(date_str, "%Y-%m-%d")

            worksheet.write_string(row, col, item)
            worksheet.write_datetime(row, col + 1, date, date_format)
            worksheet.write_number(row, col + 2, cost, money_format)
            row += 1

        # Write a total using a formula.
        worksheet.write(row, 0, "Total", bold)
        worksheet.write(row, 2, "=SUM(C2:C5)", money_format, 1450)

        workbook.close()

        self.assertExcelEqual()



###############################################################################
#
# Tests for XlsxWriter.
#
# SPDX-License-Identifier: BSD-2-Clause
# Copyright (c), 2013-2023, John McNamara, jmcnamara@cpan.org
#

import unittest
from datetime import date, time
from io import StringIO
from ...worksheet import Worksheet
from ..helperfunctions import _xml_to_list


class TestWriteDataValidations(unittest.TestCase):
    """
    Test the Worksheet _write_data_validations() method.

    """

    def setUp(self):
        self.maxDiff = None
        self.fh = StringIO()
        self.worksheet = Worksheet()
        self.worksheet._set_filehandle(self.fh)

    def test_write_data_validations_1(self):
        """
        Test 1 Integer between 1 and 10.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_2(self):
        """
        Test 2 Integer not between 1 and 10.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "not between",
                "minimum": 1,
                "maximum": 10,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" operator="notBetween" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_3(self):
        """
        Test 3,4,5 Integer == 1.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "equal to",
                "value": 1,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" operator="equal" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_4(self):
        """
        Test 3,4,5 Integer == 1.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "=",
                "value": 1,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" operator="equal" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_5(self):
        """
        Test 3,4,5 Integer == 1.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "==",
                "value": 1,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" operator="equal" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_6(self):
        """
        Test 6,7,8 Integer != 1.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "not equal to",
                "value": 1,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" operator="notEqual" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_7(self):
        """
        Test 6,7,8 Integer != 1.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "<>",
                "value": 1,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" operator="notEqual" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_8(self):
        """
        Test 6,7,8 Integer != 1.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "!=",
                "value": 1,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" operator="notEqual" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_9(self):
        """
        Test 9,10 Integer > 1.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "greater than",
                "value": 1,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" operator="greaterThan" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_10(self):
        """
        Test 9,10 Integer > 1.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": ">",
                "value": 1,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" operator="greaterThan" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_11(self):
        """
        Test 11,12 Integer < 1.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "less than",
                "value": 1,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" operator="lessThan" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_12(self):
        """
        Test 11,12 Integer < 1.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "<",
                "value": 1,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" operator="lessThan" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_13(self):
        """
        Test 13,14 Integer >= 1.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "greater than or equal to",
                "value": 1,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" operator="greaterThanOrEqual" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_14(self):
        """
        Test 13,14 Integer >= 1.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": ">=",
                "value": 1,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" operator="greaterThanOrEqual" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_15(self):
        """
        Test 15,16 Integer <= 1.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "less than or equal to",
                "value": 1,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" operator="lessThanOrEqual" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_16(self):
        """
        Test 15,16 Integer <= 1.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "<=",
                "value": 1,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" operator="lessThanOrEqual" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_17(self):
        """
        Test 17 Integer between 1 and 10 (same as test 1) + Ignore blank off.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
                "ignore_blank": 0,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_18(self):
        """
        Test 18 Integer between 1 and 10 (same as test 1) + Error style == warning.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
                "error_type": "warning",
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" errorStyle="warning" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_19(self):
        """
        Test 19 Integer between 1 and 10 (same as test 1) + Error style == info.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
                "error_type": "information",
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" errorStyle="information" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_20(self):
        """
        Test 20 Integer between 1 and 10 (same as test 1)
                + input title.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
                "input_title": "Input title January",
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" allowBlank="1" showInputMessage="1" showErrorMessage="1" promptTitle="Input title January" sqref="B5"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_21(self):
        """
        Test 21 Integer between 1 and 10 (same as test 1)
                + input title.
                + input message.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
                "input_title": "Input title January",
                "input_message": "Input message February",
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" allowBlank="1" showInputMessage="1" showErrorMessage="1" promptTitle="Input title January" prompt="Input message February" sqref="B5"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_22(self):
        """
        Test 22 Integer between 1 and 10 (same as test 1)
                + input title.
                + input message.
                + error title.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
                "input_title": "Input title January",
                "input_message": "Input message February",
                "error_title": "Error title March",
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" allowBlank="1" showInputMessage="1" showErrorMessage="1" errorTitle="Error title March" promptTitle="Input title January" prompt="Input message February" sqref="B5"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_23(self):
        """
        Test 23 Integer between 1 and 10 (same as test 1)
                + input title.
                + input message.
                + error title.
                + error message.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
                "input_title": "Input title January",
                "input_message": "Input message February",
                "error_title": "Error title March",
                "error_message": "Error message April",
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" allowBlank="1" showInputMessage="1" showErrorMessage="1" errorTitle="Error title March" error="Error message April" promptTitle="Input title January" prompt="Input message February" sqref="B5"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_24(self):
        """
        Test 24 Integer between 1 and 10 (same as test 1)
                + input title.
                + input message.
                + error title.
                + error message.
                - input message box.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
                "input_title": "Input title January",
                "input_message": "Input message February",
                "error_title": "Error title March",
                "error_message": "Error message April",
                "show_input": 0,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" allowBlank="1" showErrorMessage="1" errorTitle="Error title March" error="Error message April" promptTitle="Input title January" prompt="Input message February" sqref="B5"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_25(self):
        """
        Test 25 Integer between 1 and 10 (same as test 1)
                + input title.
                + input message.
                + error title.
                + error message.
                - input message box.
                - error message box.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
                "input_title": "Input title January",
                "input_message": "Input message February",
                "error_title": "Error title March",
                "error_message": "Error message April",
                "show_input": 0,
                "show_error": 0,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" allowBlank="1" errorTitle="Error title March" error="Error message April" promptTitle="Input title January" prompt="Input message February" sqref="B5"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_26(self):
        """
        Test 26 'Any' shouldn't produce a DV record if there are no messages.
        """
        self.worksheet.data_validation("B5", {"validate": "any"})

        self.worksheet._write_data_validations()

        exp = ""
        got = self.fh.getvalue()

        self.assertEqual(got, exp)

    def test_write_data_validations_27(self):
        """
        Test 27 Decimal = 1.2345
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "decimal",
                "criteria": "==",
                "value": 1.2345,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="decimal" operator="equal" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1.2345</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_28(self):
        """
        Test 28 List = a,bb,ccc
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "list",
                "source": ["a", "bb", "ccc"],
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="list" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>"a,bb,ccc"</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_29(self):
        """
        Test 29 List = a,bb,ccc, No dropdown
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "list",
                "source": ["a", "bb", "ccc"],
                "dropdown": 0,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="list" allowBlank="1" showDropDown="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>"a,bb,ccc"</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_30(self):
        """
        Test 30 List = $D$1:$D$5
        """
        self.worksheet.data_validation(
            "A1:A1",
            {
                "validate": "list",
                "source": "=$D$1:$D$5",
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="list" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="A1"><formula1>$D$1:$D$5</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_31(self):
        """
        Test 31 Date = 39653 (2008-07-24)
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "date",
                "criteria": "==",
                "value": date(2008, 7, 24),
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="date" operator="equal" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>39653</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_32(self):
        """
        Test 32 Date = 2008-07-25T
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "date",
                "criteria": "==",
                "value": date(2008, 7, 25),
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="date" operator="equal" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>39654</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_33(self):
        """
        Test 33 Date between ranges.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "date",
                "criteria": "between",
                "minimum": date(2008, 1, 1),
                "maximum": date(2008, 12, 12),
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="date" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>39448</formula1><formula2>39794</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_34(self):
        """
        Test 34 Time = 0.5 (12:00:00)
        """
        self.worksheet.data_validation(
            "B5:B5",
            {
                "validate": "time",
                "criteria": "==",
                "value": time(12),
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="time" operator="equal" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>0.5</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_35(self):
        """
        Test 35 Time = T12:00:00
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "time",
                "criteria": "==",
                "value": time(12, 0, 0),
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="time" operator="equal" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>0.5</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_36(self):
        """
        Test 36 Custom == 10.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "custom",
                "criteria": "==",
                "value": 10,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="custom" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>10</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_37(self):
        """
        Test 37 Check the row/col processing: single A1 style cell.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_38(self):
        """
        Test 38 Check the row/col processing: single A1 style range.
        """
        self.worksheet.data_validation(
            "B5:B10",
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5:B10"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_39(self):
        """
        Test 39 Check the row/col processing: single (row, col) style cell.
        """
        self.worksheet.data_validation(
            4,
            1,
            4,
            1,
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_40(self):
        """
        Test 40 Check the row/col processing: single (row, col) style range.
        """
        self.worksheet.data_validation(
            4,
            1,
            9,
            1,
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5:B10"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_41(self):
        """
        Test 41 Check the row/col processing: multiple (row, col) style cells.
        """
        self.worksheet.data_validation(
            4,
            1,
            4,
            1,
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
                "other_cells": [[4, 3, 4, 3]],
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5 D5"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_42(self):
        """
        Test 42 Check the row/col processing: multiple (row, col) style cells.
        """
        self.worksheet.data_validation(
            4,
            1,
            4,
            1,
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
                "other_cells": [[6, 1, 6, 1], [8, 1, 8, 1]],
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5 B7 B9"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_43(self):
        """
        Test 43 Check the row/col processing: multiple (row, col) style cells.
        """
        self.worksheet.data_validation(
            4,
            1,
            8,
            1,
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
                "other_cells": [[3, 3, 3, 3]],
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5:B9 D4"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_44(self):
        """
        Test 44 Multiple validations.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "integer",
                "criteria": ">",
                "value": 10,
            },
        )

        self.worksheet.data_validation(
            "C10",
            {
                "validate": "integer",
                "criteria": "<",
                "value": 10,
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="2"><dataValidation type="whole" operator="greaterThan" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>10</formula1></dataValidation><dataValidation type="whole" operator="lessThan" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="C10"><formula1>10</formula1></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_45(self):
        """
        Test 45 Test 'any' with input messages.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "any",
                "input_title": "Input title January",
                "input_message": "Input message February",
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation allowBlank="1" showInputMessage="1" showErrorMessage="1" promptTitle="Input title January" prompt="Input message February" sqref="B5"/></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_46(self):
        """
        Test 46 Date between ranges with formulas.
        """
        self.worksheet.data_validation(
            "B5",
            {
                "validate": "date",
                "criteria": "between",
                "minimum": date(2018, 1, 1),
                "maximum": "=TODAY()",
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="date" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5"><formula1>43101</formula1><formula2>TODAY()</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_47(self):
        """
        Test 47 Check multi range with A1 style cell ranges.
        """
        self.worksheet.data_validation(
            4,
            1,
            9,
            1,
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
                "multi_range": "B5:B10 D5:D10",
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5:B10 D5:D10"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)

    def test_write_data_validations_48(self):
        """
        Test 48 Check multi range with A1 style cells.
        """
        self.worksheet.data_validation(
            4,
            1,
            4,
            1,
            {
                "validate": "integer",
                "criteria": "between",
                "minimum": 1,
                "maximum": 10,
                "other_cells": [[4, 3, 4, 3]],
                "multi_range": "B5 C5",
            },
        )

        self.worksheet._write_data_validations()

        exp = '<dataValidations count="1"><dataValidation type="whole" allowBlank="1" showInputMessage="1" showErrorMessage="1" sqref="B5 C5"><formula1>1</formula1><formula2>10</formula2></dataValidation></dataValidations>'
        got = self.fh.getvalue()

        exp = _xml_to_list(exp)
        got = _xml_to_list(got)

        self.assertEqual(got, exp)
